<template>
  <div class="mod-config">
    <basic-container>
      <ttable
        :table="table"
        @handleSelectionChange="handleSelectionChange"
        :prelang="prelang"
        :options="options"
        :ajaxFun="ajaxFun"
        ref="checkTable"
      >
        <template #toolbar>
          <el-dropdown @command="handleCommand">
            <el-button>
              状态变更<i class="el-icon-arrow-down el-icon--right"></i>
            </el-button>
            <el-dropdown-menu slot="dropdown">
              <el-dropdown-item
                :command="item.command"
                v-for="item in stateTagArr"
                :key="item.label"
                :disabled="item.disabled"
                >{{ item.label }}</el-dropdown-item
              >
            </el-dropdown-menu>
          </el-dropdown>
        </template>
      </ttable>
      <!-- 弹窗, 新增 / 修改 -->
      <table-form
        v-if="addOrUpdateVisible"
        ref="addOrUpdate"
        @refreshDataList="getData"
      ></table-form>
    </basic-container>
  </div>
</template>

<script>
import { fetchList, putObj, delObj, exportMain } from '@/api/warehouse/check'
import ttable from '@/views/common/ztt-table.vue'
import { mapGetters } from 'vuex'

import TableForm from './check-form'
import { transform } from '@/util/fileTransform'

export default {
  data() {
    const col = (prop, lable, config) => {
      const w = (lable.length - 1) * 20 + 15
      return Object.assign(
        {
          minWidth: w < 140 ? 140 : w,
          prop: prop,
          label: lable,
          sort: true,
          isTrue: true,
          isSearch: true,
          searchInfoType: 'text'
        },
        config
      )
    }
    return {
      ajaxFun: fetchList,
      multipleSelection: [],
      prelang: 'operation',
      options: {
        height: 300, // 默认高度-为了表头固定
        stripe: true, // 是否为斑马纹 table
        highlightCurrentRow: false, // 是否要高亮当前行
        border: true, // 是否有纵向边框
        lazy: false, // 是否需要懒加载
        fit: true, // 列的宽度是否自撑开
        multiSelect: true, //
        seqNo: true,
        seqNoFix: true,
        isRefresh: true, // 是否显示刷新按钮
        isShowHide: true, // 是否显示显影按钮
        isSearch: false, // 高级查询按钮
        defaultOrderBy: { column: 'createTime', direction: 'desc' }
      },
      table: {
        total: 0,
        currentPage: 1,
        pageSize: 20,
        data: [],
        // 标题
        column: [
          col('inyNo', '盘点编号', {
            render: { fun: this.goDetail }
          }),
          col('inyDesc', '盘点描述'),
          col('inyStatus', '状态', {
            formatter: (row, column, cellValue) => {
              return cellValue === 'draft' ? '草稿' : '已完成'
            }
          }),
          col('createUser', '创建人'),
          col('createTime', '创建时间', {
            searchInfoType: 'datetimerange'
          }),
          col('updateTime', '更新时间', {
            searchInfoType: 'datetimerange'
          })
        ],
        toolbar: [
          {
            text: '新增',
            type: 'primary',
            fun: this.addOrUpdateHandle
          },
          {
            text: '导出',
            type: 'primary',
            fun: this.exportHandle
          }
        ],
        operator: [
          {
            text: '删除',
            type: 'text',
            size: 'small',
            fun: this.deleteHandle
          }
        ],
        operatorConfig: {
          fixed: 'right',
          label: '操作',
          width: 100
        }
      },
      stateTagArr: [
        {
          label: '草稿',
          command: 'draft',
          disabled: false,
          permitArr: []
        },
        {
          label: '已完成',
          command: 'finished',
          disabled: false,
          permitArr: []
        }
      ],
      addOrUpdateVisible: false
    }
  },
  components: {
    ttable,
    TableForm
  },
  computed: {
    ...mapGetters(['permissions'])
  },
  activated() {
    this.getData()
  },
  methods: {
    // 获取数据列表
    getData() {
      this.$refs.checkTable.getDataList()
    },
    // 新增 / 修改
    addOrUpdateHandle(row) {
      this.addOrUpdateVisible = true
      this.$nextTick(() => {
        this.$refs.addOrUpdate.init(row == null ? null : row.id)
      })
    },
    // 导出
    exportHandle() {
      if (this.multipleSelection && this.multipleSelection.length > 0) {
        exportMain({
          ids: this.multipleSelection.map((item) => {
            return item.id
          })
        })
          .then((response) => {
            transform(response)
          })
          .catch(() => {})
      } else {
        this.$message.error('请选择盘点编号')
      }
    },
    // 删除
    deleteHandle(row) {
      this.$confirm('是否确认盘点编号：' + row.inyNo, '提示', {
        confirmButtonText: '确定',
        cancelButtonText: '取消',
        type: 'warning'
      })
        .then(function() {
          return delObj(row.id)
        })
        .then((data) => {
          this.$message.success('删除成功')
          this.getData()
        })
    },
    // table自带事件
    handleSelectionChange(val) {
      // 根据状态，禁用表头按钮
      // 筛选出选中记录的状态
      var stateArr = val.map(function(value, index) {
        return value.state
      })
      // 选中状态数组元素去重
      var uniqueStateArr = []
      for (var i = 0; i < stateArr.length; i++) {
        if (uniqueStateArr.indexOf(stateArr[i]) == -1) {
          uniqueStateArr.push(stateArr[i])
        }
      }
      // 循环自定义按钮，判断每个按钮的permitArr是否完全包含选中状态，若完全包含，则按钮亮，否则按钮灰
      for (var i = 0; i < this.stateTagArr.length; i++) {
        if (
          uniqueStateArr.every((val) =>
            this.stateTagArr[i].permitArr.length <= 0
              ? true
              : this.stateTagArr[i].permitArr.includes(val)
          )
        ) {
          this.stateTagArr[i].disabled = false
        } else {
          this.stateTagArr[i].disabled = true
        }
      }
      this.multipleSelection = val
    },
    // 状态变更
    handleCommand(event) {
      if (this.multipleSelection && this.multipleSelection.length > 0) {
        const p = []

        this.multipleSelection.forEach((item) => {
          p.push(
            putObj(
              Object.assign(
                {
                  ...item
                },
                {
                  inyStatus: event
                }
              )
            )
          )
        })
        Promise.all(p).then(() => {
          this.$message.success('状态变更成功')
          this.getData()
        })
      } else {
        this.$message.error('请选择盘点编号')
      }
    },
    goDetail(row) {
      this.$router.push({
        name: 'checkDetail',
        query: {
          id: row.id,
          inyNo: row.inyNo,
          canEdit: row.inyStatus === 'draft'
        }
      })
    }
  }
}
</script>
